package io.p2vman.pilers;

import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap;
import lombok.experimental.Delegate;
import org.bukkit.plugin.Plugin;
import org.bukkit.scheduler.BukkitScheduler;
import org.bukkit.scheduler.BukkitTask;

import java.util.Map;
import java.util.concurrent.CompletableFuture;

public class TaskMannager implements BukkitScheduler {
    @Delegate
    private final BukkitScheduler scheduler;
    private final Plugin plugin;
    private final Map<String, BukkitTask> tasks = new Object2ObjectOpenHashMap<>();
    public TaskMannager(BukkitScheduler scheduler, Plugin plugin) {
        this.scheduler = scheduler;
        this.plugin = plugin;
    }

    public boolean isRunning(String id) {
        return tasks.containsKey(id) && !tasks.get(id).isCancelled();
    }

    public void cancel(String id) {
        BukkitTask task = tasks.remove(id);
        if (task != null) task.cancel();
    }

    public void cancel(String... ids) {
        for (String id : ids) {
            BukkitTask task = tasks.remove(id);
            if (task != null) task.cancel();
        }
    }

    public void cancelAll() {
        for (BukkitTask task : tasks.values()) {
            task.cancel();
        }
        tasks.clear();
    }

    public void runLater(String id, Runnable runnable, long delayTicks) {
        cancel(id);
        BukkitTask task = runTaskLater(plugin, runnable, delayTicks);
        tasks.put(id, task);
    }

    public void runRepeating(String id, Runnable runnable, long delayTicks, long periodTicks) {
        cancel(id);
        BukkitTask task = runTaskTimer(plugin, runnable, delayTicks, periodTicks);
        tasks.put(id, task);
    }

    public CompletableFuture<Void> runAsync(String id, Runnable runnable) {
        cancel(id);
        CompletableFuture<Void> future = new CompletableFuture<>();
        BukkitTask task = runTaskAsynchronously(plugin, () -> {
            try {
                runnable.run();
                future.complete(null);
            } catch (Exception e) {
                future.completeExceptionally(e);
            }
        });
        tasks.put(id, task);
        return future;
    }

    public void runAsyncRepeating(String id, Runnable runnable, long delayTicks, long periodTicks) {
        cancel(id);
        BukkitTask task = runTaskTimerAsynchronously(plugin, runnable, delayTicks, periodTicks);
        tasks.put(id, task);
    }
}
